<?php
namespace InternetGalerie\Igshop2\Controller;

use Ig\IgPayment\Services\PostFinanceCheckoutService;
use InternetGalerie\Igshop2\Domain\Model\Order;
use InternetGalerie\Igshop2\Domain\Repository\CountryRepository;
use PostFinanceCheckout\Sdk\Model\Gender;
use PostFinanceCheckout\Sdk\Model\LineItemCreate;
use PostFinanceCheckout\Sdk\Model\LineItemType;
use Psr\Http\Message\ResponseInterface;
use TYPO3\CMS\Core\Http\RedirectResponse;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * PostFinanceCheckoutController
 */
class PostFinanceCheckoutController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
    protected PostFinanceCheckoutService $pfCheckoutService;
    protected CountryRepository $countryRepository;

    public function __construct(CountryRepository $countryRepository)
    {
        $this->countryRepository = $countryRepository;
    }
    
    public function initializeAction(): void
    {
        $this->pfCheckoutService = GeneralUtility::makeInstance(PostFinanceCheckoutService::class);
    }

    protected function getAddressArray(array $data = []): array
    {
        $salutation = '';
        $country = null;
        $countryName = '';

        if(isset($data['salutation']) && $data['salutation']) {
            $salutation = $data['salutation'] == 1 ? Gender::MALE : Gender::FEMALE;
        }

        if (isset($data['country']) && $data['country']) {
            $country = $this->countryRepository->findByUid($data['country']);
            if ($country && $country->getDataTrigger()) {
                $countryName = $data['foreignCountry'];
            } else if($country) {
                $countryName = $country->getName();
            }
        }

        $address = [
            'gender' => $salutation,
            'given_name' => $data['firstName'] ?? null,
            'family_name' => $data['lastName'] ?? null,
            'organization_name' => $data['company'] ?? null,
            'street' => $data['address'] ?? null,
            'postcode' => $data['zip'] ?? null,
            'city' => $data['city'] ?? null,
            'country' => $countryName,
            'phone_number' => $data['phone'] ?? null,
            'email_address' => $data['mail'] ?? null,
        ];

        return $address;
    }

    public function transactionAction(Order $order): ResponseInterface
    {
        if($order->getSubmitted() > 0) {
            return $this->redirect('review', 'Order', null, null, $this->settings['cartPid']);
        }
        
        $lineItems = [];
        foreach($order->getProducts() as $orderProduct) {
            $orderProduct->calcPrice($order->getCurrency());
            $product = $orderProduct->getProduct();
            $lineItem = new LineItemCreate();
            $lineItem->setName($product->getName())
                     ->setUniqueId('Product-' . $product->getUid())
                     ->setQuantity($orderProduct->getAmount())
                     ->setAmountIncludingTax($orderProduct->getPrice())
                     ->setType(LineItemType::PRODUCT);
            
            $lineItems[] = $lineItem;
        }

        if($order->getPorto()) {
            $porto = $order->getPorto();
            $lineItem = new LineItemCreate();
            $lineItem->setName($porto->getName())
                     ->setUniqueId('Porto-' . $porto->getUid())
                     ->setQuantity(1)
                     ->setAmountIncludingTax($porto->getPrice())
                     ->setType(LineItemType::SHIPPING);

            $lineItems[] = $lineItem;
        }

        $pfCheckoutSettings = $this->pfCheckoutService->getSettings();

        $successUrl = $this->uriBuilder
            ->reset()
            ->setTargetPageUid((int)$pfCheckoutSettings['pfCheckout']['pfCheckoutHandlerPid'])
            ->setCreateAbsoluteUri(true)
            ->uriFor('success', ['order' => $order->getUid()], 'PostFinanceCheckout', 'Igshop2', 'PostFinanceCheckoutHandler');

        $failUrl = $this->uriBuilder
            ->reset()
            ->setTargetPageUid((int)$pfCheckoutSettings['pfCheckout']['pfCheckoutHandlerPid'])
            ->setCreateAbsoluteUri(true)
            ->uriFor('fail', ['order' => $order->getUid()], 'PostFinanceCheckout', 'Igshop2', 'PostFinanceCheckoutHandler');

        /** @var array $orderInfo */
        $orderInfo = $order->getOrderinfo();

        $shippingAddress = $this->getAddressArray($orderInfo);
        $billingAddress = $shippingAddress;

        if($orderInfo['isCalcAddress'] || $orderInfo['extraBillingAddress']) {
            $billingAddress = $this->getAddressArray($orderInfo['calcAddress']);
        }
        
        $paymentPageUrl = $this->pfCheckoutService->getPaymentPageUrl(
            $order->getUid(),
            $shippingAddress,
            $billingAddress,
            $lineItems,
            $order->getCurrency()->getShort(),
            $successUrl,
            $failUrl
        );

        return new RedirectResponse($paymentPageUrl);
    }

    public function successAction(Order $order): ResponseInterface
    {        
        return $this->redirect('submitPayment', 'Order', null, ['complus' => 1, 'ordertosubmit' => $order->getUid()], $this->settings['cartPid']);
    }

    public function failAction(): ResponseInterface
    {
        return $this->redirect('review', 'Order', null, null, $this->settings['cartPid']);
    }
    
}
