<?php
namespace InternetGalerie\Igshop2\Domain\Model;


/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2017 FS <fs@internetgalerie.ch>, Internetgalerie
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * FeUser
 */
class FeUser extends \Internetgalerie\IgFrontendUser\Domain\Model\FrontendUser
{
    /**
     * @var \InternetGalerie\Igshop2\Domain\Model\Address
     */
    protected $lastUsedAddress = null;

    /**
     * @var \InternetGalerie\Igshop2\Domain\Model\Address
     */
    protected $lastUsedBillingAddress = null;

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Address>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade remove
     */
    protected $addresses = null;

    /**
     * @var \InternetGalerie\Igshop2\Domain\Model\Order
     */
    protected $currentOrder = null;

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Order>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $orders = null;

    /**
     * Constructor
     */
    public function __construct() {
        $this->initStorageObjects();
    }

    /**
     * Initialize ObjectStorages
     */
    protected function initStorageObjects() {
        $this->addresses = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->orders = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * @return  \InternetGalerie\Igshop2\Domain\Model\Address $lastUsedAddress
     */
    public function getLastUsedAddress(){
        return $this->lastUsedAddress;
    }

    /**
     * @param \InternetGalerie\Igshop2\Domain\Model\Address $lastUsedAddress
     */
    public function setLastUsedAddress($lastUsedAddress){
        $this->lastUsedAddress = $lastUsedAddress;
    }

    /**
     * @return \InternetGalerie\Igshop2\Domain\Model\Address $lastUsedBillingAddress
     */
    public function getLastUsedBillingAddress(){
        return $this->lastUsedBillingAddress;
    }

    /**
     * @param \InternetGalerie\Igshop2\Domain\Model\Address $lastUsedBillingAddress
     */
    public function setLastUsedBillingAddress($lastUsedBillingAddress){
        $this->lastUsedBillingAddress = $lastUsedBillingAddress;
    }

    /**
     * Returns the addresses
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Address> $addresses
     */
    public function getAddresses(){
        return $this->addresses;
    }

    /**
     * Sets the addresses
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Address> $addresses
     */
    public function setAddresses($addresses){
        $this->addresses = $addresses;
    }

    /**
     * Adds an address
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Address $address
     */
    public function addAddress($address) {
        $this->addresses->attach($address);
    }

    /**
     * Removes an address
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Address $addressToRemove
     */
    public function removeAddress($addressToRemove) {
        $this->addresses->detach($addressToRemove);
    }

    /**
     * @return \InternetGalerie\Igshop2\Domain\Model\Order $currentOrder
     */
    public function getCurrentOrder(){
        return $this->currentOrder;
    }

    /**
     * @param \InternetGalerie\Igshop2\Domain\Model\Order $currentOrder
     */
    public function setCurrentOrder($currentOrder){
        $this->currentOrder = $currentOrder;
    }

    /**
     * Returns the orders
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Order> $orders
     */
    public function getOrders(){
        return $this->orders;
    }

    /**
     * Sets the orders
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Order> $orders
     */
    public function setOrders($orders){
        $this->orders = $orders;
    }

    /**
     * Adds an order
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Order $order
     */
    public function addOrder($order) {
        $this->orders->attach($order);
    }

    /**
     * Removes an order
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Order $orderToRemove
     */
    public function removeOrder($orderToRemove) {
        $this->orders->detach($orderToRemove);
    }
}
