<?php
namespace InternetGalerie\Igshop2\Domain\Model;

/**
 * Shop
 */
class Shop extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * name
     *
     * @var string
     */
    protected $name = '';

    /**
     * countries
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Country>
     */
    protected $countries = null;

    /**
     * currencies
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Currency>
     */
    protected $currencies = null;

    /**
     * defaultCountry
     *
     * @var \InternetGalerie\Igshop2\Domain\Model\Country
     */
    protected $defaultCountry = null;

    /**
     * paymentmethods
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\PaymentMethod>
     */
    protected $paymentmethods = null;

    /**
     * portos
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Porto>
     */
    protected $portos = null;

    /**
     * defaultPorto
     *
     * @var \InternetGalerie\Igshop2\Domain\Model\Porto
     */
    protected $defaultPorto = null;

    /**
     * __construct
     */
    public function __construct()
    {

        //Do not remove the next line: It would break the functionality
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    protected function initStorageObjects()
    {
        $this->countries = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->currencies = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->paymentmethods = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->portos = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Returns the name
     *
     * @return string $name
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Sets the name
     *
     * @param string $name
     * @return void
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Adds a Country
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Country $country
     * @return void
     */
    public function addCountry(\InternetGalerie\Igshop2\Domain\Model\Country $country)
    {
        $this->countries->attach($country);
    }

    /**
     * Removes a Country
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Country $countryToRemove The Country to be removed
     * @return void
     */
    public function removeCountry(\InternetGalerie\Igshop2\Domain\Model\Country $countryToRemove)
    {
        $this->countries->detach($countryToRemove);
    }

    /**
     * Returns the countries
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Country> $countries
     */
    public function getCountries()
    {
        return $this->countries;
    }

    /**
     * Sets the countries
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Country> $countries
     * @return void
     */
    public function setCountries(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $countries)
    {
        $this->countries = $countries;
    }

    /**
     * Adds a Currency
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Currency $currency
     * @return void
     */
    public function addCurrency(\InternetGalerie\Igshop2\Domain\Model\Currency $currency)
    {
        $this->currencies->attach($currency);
    }

    /**
     * Removes a Currency
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Currency $currencyToRemove The Currency to be removed
     * @return void
     */
    public function removeCurrency(\InternetGalerie\Igshop2\Domain\Model\Currency $currencyToRemove)
    {
        $this->currencies->detach($currencyToRemove);
    }

    /**
     * Returns the currencies
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Currency> $currencies
     */
    public function getCurrencies()
    {
        return $this->currencies;
    }

    /**
     * Sets the currencies
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Currency> $currencies
     * @return void
     */
    public function setCurrencies(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $currencies)
    {
        $this->currencies = $currencies;
    }

    public function getCurrenciesAssociative()
    {
        $currenciesAssociative = [];
        foreach($this->currencies as $currency) {
            $currenciesAssociative[$currency->getShort()] = $currency;
        }
        return $currenciesAssociative;
    }

    /**
     * Returns the defaultCountry
     *
     * @return \InternetGalerie\Igshop2\Domain\Model\Country $defaultCountry
     */
    public function getDefaultCountry()
    {
        return $this->defaultCountry;
    }

    /**
     * Sets the defaultCountry
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Country $defaultCountry
     * @return void
     */
    public function setDefaultCountry(\InternetGalerie\Igshop2\Domain\Model\Country $defaultCountry)
    {
        $this->defaultCountry = $defaultCountry;
    }

    /**
     * Adds a PaymentMethod
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\PaymentMethod $paymentmethod
     * @return void
     */
    public function addPaymentMethod(\InternetGalerie\Igshop2\Domain\Model\PaymentMethod $paymentmethod)
    {
        $this->paymentmethods->attach($paymentmethod);
    }

    /**
     * Removes a PaymentMethod
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\PaymentMethod $paymentmethodToRemove The PaymentMethod to be removed
     * @return void
     */
    public function removePaymentMethod(\InternetGalerie\Igshop2\Domain\Model\PaymentMethod $paymentmethodToRemove)
    {
        $this->paymentmethods->detach($paymentmethodToRemove);
    }

    /**
     * Returns the paymentmethods
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\PaymentMethod> $paymentmethods
     */
    public function getPaymentMethods()
    {
        return $this->paymentmethods;
    }

    /**
     * Sets the paymentmethods
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\PaymentMethod> $paymentmethods
     * @return void
     */
    public function setPaymentMethods(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $paymentmethods)
    {
        $this->paymentmethods = $paymentmethods;
    }

    /**
     * Adds a Porto
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Porto $porto
     * @return void
     */
    public function addPorto(\InternetGalerie\Igshop2\Domain\Model\Porto $porto)
    {
        $this->portos->attach($porto);
    }

    /**
     * Removes a Porto
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Porto $portoToRemove The Porto to be removed
     * @return void
     */
    public function removePorto(\InternetGalerie\Igshop2\Domain\Model\Porto $portoToRemove)
    {
        $this->portos->detach($portoToRemove);
    }

    /**
     * Returns the portos
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Porto> $portos
     */
    public function getPortos()
    {
        return $this->portos;
    }

    /**
     * Returns the portos
     *
     * @return array
     */
    public function getPortosForCountry($country)
    {
        if( !is_object($country)) {
            return null;
        }
        /*
		$query = $this->createQuery();
		// @TODO something is completely broken DA [multilanguage sites]
		$query->getQuerySettings()->setRespectSysLanguage(false);
		$query->matching(
            $query->logicalOr(
                $query->contains('country',$country),
                $query->equals('countryAll', true)
            )
        );
		return $query->executeQuery();
        */

        $portos = [];
        foreach( $this->portos as $porto) {
            if($porto->getCountryAll() || $porto->getCountry()==$country) {
                $portos[] = $porto;
            }
        }
        return $portos;
    }

    /**
     * Sets the portos
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\InternetGalerie\Igshop2\Domain\Model\Porto> $portos
     * @return void
     */
    public function setPortos(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $portos)
    {
        $this->portos = $portos;
    }

    /**
     * Returns the defaultPorto
     *
     * @return \InternetGalerie\Igshop2\Domain\Model\Porto $defaultPorto
     */
    public function getDefaultPorto()
    {
        return $this->defaultPorto;
    }
    /**
     * Returns the defaultPorto
     *
     * @return \InternetGalerie\Igshop2\Domain\Model\Porto $defaultPorto
     */
    public function getDefaultPortoForCountry($country)
    {
        if($this->defaultPorto->hasCountry($country)) {
            return $this->defaultPorto;
        }
        // @todo analog altem Code
        //$countryPortos = $this->getPortosForCountry($country);
        //return $countryPortos->getFirst();
        return null;
    }

    /**
     * Sets the defaultPorto
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\Porto $defaultPorto
     * @return void
     */
    public function setDefaultPorto(\InternetGalerie\Igshop2\Domain\Model\Porto $defaultPorto)
    {
        $this->defaultPorto = $defaultPorto;
    }
}
