<?php

namespace InternetGalerie\Igshop2\ViewHelpers;

/**
 * This class is a demo view helper for the Fluid templating engine.
 *
 * @package TYPO3
 * @subpackage Fluid
 * @version
 */
class CurrencyViewHelper extends \TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper {

 /**
     * As this ViewHelper renders HTML, the output must not be escaped.
     *
     * @var bool
     */
    protected $escapeOutput = false;

    public function initializeArguments(): void
	{
        parent::initializeArguments();
        $this->registerArgument('currency', \InternetGalerie\Igshop2\Domain\Model\Currency::class, 'Specifies where to open the linked document', true);
        $this->registerArgument('price', 'float', 'Specifies where to open the linked document', false, 0);
        $this->registerArgument('withoutCode', 'boolean', 'Specifies where to open the linked document', false, false);
        $this->registerArgument('product', \InternetGalerie\Igshop2\Domain\Model\Product::class, 'Specifies where to open the linked document', false, null);
        $this->registerArgument('optionWithPrice', \InternetGalerie\Igshop2\Domain\Model\OptionWithPrice::class, 'Specifies where to open the linked document', false, null);
        $this->registerArgument('checkForPromotion', 'boolean', 'Specifies where to open the linked document', false, false);
        $this->registerArgument('wrap', 'boolean', 'Wrap around price', false, false);
    }

    public function renderByArguments($currency, $price = 0, $withoutCode = false, $product = null, $optionWithPrice = null, $checkForPromotion = false): string
	{
        $this->arguments["currency"] = $currency;
        $this->arguments["price"] = $price;
        $this->arguments["withoutCode"] = $withoutCode;
        $this->arguments["product"] = $product;
        $this->arguments["optionWithPrice"] = $optionWithPrice;
        $this->arguments["checkForPromotion"] = $checkForPromotion;
        return $this->render();
    }

    /**
     * Rounds the price to CHF norm
	 *
	 * @return string
     */
	public function render(): string
	{
        $currency = $this->arguments["currency"];
		$price = $this->arguments["price"];
		$withoutCode = $this->arguments["withoutCode"];
        $product = $this->arguments["product"];
        $optionWithPrice = $this->arguments["optionWithPrice"];
		$checkForPromotion = $this->arguments["checkForPromotion"];
		$wrap = $this->arguments["wrap"];

		if(is_object($optionWithPrice)){
			if($currency->getShort() == "EUR" && $optionWithPrice->getPriceEuro() !== null){
				$price = $optionWithPrice->getPriceEuro();
			}else{
				$price = $optionWithPrice->getPrice() * $currency->getFaktor();
			}
		}

		if(is_object($product)){
			$price = $product->getFinalPrice($currency, 0,0, $checkForPromotion);
		}
		//var_dump($price);
		if(is_array($price)){
			$promotePrice = $this->format($currency, $price["price"], $withoutCode);
			$origPrice = $this->format($currency, $price["origPrice"], $withoutCode);
			return '<div class="origPrice hasPromoPrice">'.$origPrice.'</div><div class="promoPrice">'.$promotePrice.'</div>';
		}else{
			$price = ($wrap ? '<div class="noPromoPrice">' : '') . $this->format($currency, $price, $withoutCode) . ($wrap ? '</div>' : '');
		}

		return $price;
	}

	static function format($currency, $price, $withoutCode): string
	{
        $thousandSeparator = $currency->getThousandSeparator();
        $commaSeparator = $currency->getCommaSeparator();
		$return = 0;

		if($price) {
			if($currency->getRounding()){
				$rounding = $currency->getRounding();
				if($thousandSeparator != null && $commaSeparator  != null){
					$return = number_format(round($price*(1/$rounding))/(1/$rounding),2,$commaSeparator, $thousandSeparator);
				}else{
					$return = number_format(round($price*(1/$rounding))/(1/$rounding),2);
				}
			}else{
				if($thousandSeparator != null && $commaSeparator  != null){
					$return = number_format($price,2,$commaSeparator, $thousandSeparator);
				}else{
					$return = number_format($price,2);
				}
			}
		}

		if($withoutCode){
			return $return;
		}

		if($currency->getAfterPrice()){
			return $return." ".$currency->getSymbol();
		}elseif($return > 0) {
            return $currency->getShort()." ".$return;
        }else{
            return \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_igshop2_domain_model_product.free', 'igshop2');
		}
	}
}

?>
